<?php

declare(strict_types=1);

namespace Drupal\hubspot_connect\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Hubspot connect settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'hubspot_connect_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['hubspot_connect.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('hubspot_connect.settings');

    $form['account'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('General settings'),
    ];

    $form['account']['hubspot_tracking_code'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Tracking Code'),
      '#default_value' => $config->get('hubspot_tracking_code'),
      '#size' => 15,
      '#maxlength' => 10,
      '#description' => $this->t('<a href="@hubspot">The HubSpot tracking code</a> is unique to each HubSpot account and allows HubSpot to monitor your website traffic.', [
        '@hubspot' => 'https://knowledge.hubspot.com/reports/install-the-hubspot-tracking-code',
      ]),
    ];

    $form['exclude'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Excludes'),
    ];

    $roles = $this->entityTypeManager
      ->getStorage('user_role')
      ->loadMultiple();
    $role_options = [];
    foreach ($roles as $role) {
      $role_options[$role->id()] = $role->label();
    }

    $form['exclude']['exclude_roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#default_value' => $config->get('exclude_roles') ?? [],
      '#description' => $this->t('The Hubspot tracking will be disabled for selected roles.'),
      '#options' => $role_options,
    ];

    $form['exclude']['exclude_paths'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Path patterns'),
      '#default_value' => $config->get('exclude_paths') ?? '',
      '#description' => $this->t('The Hubspot tracking will be disabled for selected paths patterns, like "/admin/*". Put one path to each row.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $hubspot_tracking_code = trim($form_state->getValue('hubspot_tracking_code'));
    if (!preg_match('/^\d+$/', $hubspot_tracking_code)) {
      $form_state->setErrorByName(
        'hubspot_tracking_code',
        $this->t('A valid HubSpot tracking code contains only digits and formatted like xxxxxxx.'),
      );
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('hubspot_connect.settings')
      ->set('hubspot_tracking_code', trim($form_state->getValue('hubspot_tracking_code')))
      ->set('exclude_roles', array_filter($form_state->getValue('exclude_roles')))
      ->set('exclude_paths', trim($form_state->getValue('exclude_paths')))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
